import React, { Component } from 'react'
import { message, Modal, notification } from 'antd'
import md5 from 'md5'
import moment from 'moment'

import Api from '@/api'
import Utils from '@/utils/utils.js' 
import MKEmitter from '@/utils/events.js'
import options, { styles } from '@/store/options.js'
import zhCN from '@/locales/zh-CN/login.js'
import enUS from '@/locales/en-US/login.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncLoadComponent from '@/utils/asyncLoadComponent'
import './index.scss'

const LoginForm = asyncLoadComponent(() => import('./loginform'))
const Resetpwd = asyncLoadComponent(() => import('@/components/resetPassword'))
const LoginCloudForm = asyncComponent(() => import('./logincloudform'))
const LoginCodeForm = asyncComponent(() => import('./logincodeform'))
const iszhCN = sessionStorage.getItem('lang') !== 'en-US'

const _href = window.location.href.split('#')[0]

class Login extends Component {
  state = {
    selectedlang: iszhCN ? 'zh-CN' : 'en-US',
    dict: iszhCN ? zhCN : enUS,
    isDisabled: false,
    auth: false,
    authError: '',
    loginlogo: window.GLOB.loginlogo || '',
    bgImage: window.GLOB.bgImage || '',
    platName: window.GLOB.platName || '',
    copyRight: window.GLOB.copyRight || '',
    ICP: window.GLOB.ICP || '',
    lineColor: window.GLOB.lineColor || '',
    webSite: window.GLOB.webSite || '',
    langList: [],
    syncApp: false,
    loginWays: null,
    touristLogin: false,
    syncing: false,
  }

  UNSAFE_componentWillMount() {
    localStorage.removeItem('UserID')
    localStorage.removeItem('LoginUID')
    localStorage.removeItem('User_Name')
    localStorage.removeItem('Full_Name')
    localStorage.removeItem('avatar')
    localStorage.removeItem('dataM')
    localStorage.removeItem('localDataM')
    localStorage.removeItem('debug')
    localStorage.removeItem('role_id')

    sessionStorage.clear()
  }

  changelang (item) {
    // 切换语言
    this.setState({
      dict: item === 'zh-CN' ? zhCN : enUS
    })
  }

  handleSubmit = () => {
    this.loginformRef.handleConfirm().then(res => {
      this.setState({
        isDisabled: true
      })

      if (res.type === 'uname_pwd') {
        this.loginsubmit(res)
      } else if (res.type === 'sms_vcode') {
        this.phoneloginsubmit(res)
      }
    })
  }

  /**
   * @description 账号密码登录
   * @param {Object} param 用户名密码等信息
   */
  async loginsubmit (param) {
    // 登录提交
    let res = await Api.getusermsg(param.username, param.password, false)
    if (res.status) {
      if (res.check_mob) {
        let loginWays = this.state.loginWays.filter(item => item.type === 'sms_vcode')

        if (loginWays.length > 0) {
          this.setState({loginWays: null, isDisabled: false}, () => {
            this.setState({loginWays: loginWays})
          })
          Modal.warning({
            title: '为确认是您本人操作，请使用手机短信验证码进行身份认证',
            okText: '知道了'
          })
          return
        }
      }
      localStorage.setItem('UserID', res.UserID)
      localStorage.setItem('LoginUID', res.LoginUID)
      localStorage.setItem('User_Name', res.UserName)
      localStorage.setItem('Full_Name', res.FullName)
      localStorage.setItem('avatar', res.icon || '')
      localStorage.setItem('dataM', res.dataM ? 'true' : '')
      localStorage.setItem('localDataM', res.dataM ? 'true' : '')
      localStorage.setItem('debug', res.debug || '')
      localStorage.setItem('role_id', res.role_id || '')
      localStorage.setItem('departmentcode', res.departmentcode || '')
      localStorage.setItem('organization', res.organization || '')
      localStorage.setItem('mk_user_type', res.mk_user_type || '')

      localStorage.setItem('lang', 'zh-CN')

      let _url = window.location.href.split('#')[0]

      // 记住密码时账号密码存入localStorage
      localStorage.setItem(_url, window.btoa(window.encodeURIComponent(JSON.stringify({username: param.username, password: param.password}))))

      let _param = {
        UserName: param.username,
        Password: param.password,
        BasePath: '/'
      }
      
      // positecgroup
      if (res.token) {
        this.setState({
          isDisabled: false,
          verify: {...res, $password: param.password},
          codeVisible: true
        })
        return
      }
      
      let level = localStorage.getItem(_href + 'pwdlevel')

      if (level && process.env.NODE_ENV === 'production') {
        let visible = false
        let tip = '密码强度不够，请修改密码！'
        if (param.password.length < 8) {
          visible = true
        } else if (level === 'letter_num' && /^([^0-9]*|[^a-zA-Z]*)$/.test(param.password)) {
          visible = true
        } else if ((level === 'char_num' || level === 'char_num_90' || level === 'char_num_90_sms') && /^([^0-9]*|[^a-zA-Z]*|[^!@#$%^&*()_]*)$/.test(param.password)) {
          visible = true
        } else if ((level === 'char_num_90' || level === 'char_num_90_sms') && res.modifydate) {
          let s = (new Date().getTime() - new Date(res.modifydate).getTime()) / (1000 * 24 * 60 * 60)
          if (!isNaN(s) && s > 90) {
            visible = true
            tip = '您已90天未修改密码，请更换密码后使用！'
          }
        }

        if (visible) {
          message.warning(tip)
          this.setState({
            isDisabled: false
          })
          
          MKEmitter.emit('resetpassword', () => {
            const input = document.getElementById('password')
            if (input) {
              input.select()
            }
          })
          return
        }
      }

      if (process.env.NODE_ENV === 'production') {
        Api.loginAndRedirect(_param).then(result => {
          if (result.IsError) {
            this.setState({
              isDisabled: false
            })
            message.warning('登录失败,用户名或密码错误!')
          } else {
            window.location.replace(document.location.origin + '/zh-CN/Home/Index')
          }
        })
      } else {
        sessionStorage.setItem('UserID', res.UserID)
        sessionStorage.setItem('LoginUID', res.LoginUID)
        sessionStorage.setItem('User_Name', res.UserName)
        sessionStorage.setItem('Full_Name', res.FullName)
        sessionStorage.setItem('avatar', res.icon || '')
        sessionStorage.setItem('dataM', res.dataM ? 'true' : '')
        sessionStorage.setItem('localDataM', res.dataM ? 'true' : '')
        sessionStorage.setItem('debug', res.debug || '')
        sessionStorage.setItem('role_id', res.role_id || '')
        sessionStorage.setItem('mk_user_type', res.mk_user_type || '')

        this.props.history.replace('/main')
      }
    } else if (res.ErrCode === 'Need_Get_Appkey' && options.sysType === 'SSO') {
      message.warning('应用尚未创建，请向云端同步应用！')

      this.setState({
        isDisabled: false,
        syncApp: true
      })
    } else {
      message.warning(res.message)
      if (res.message.indexOf('密码错误') > -1) {
        const input = document.getElementById('password')
        if (input) {
          input.select()
        }
      } else if (res.message.indexOf('登录权限') > -1) {
        const input = document.getElementById('username')
        if (input) {
          input.select()
        }
      }
      this.setState({
        isDisabled: false
      })
    }
  }

  async phoneloginsubmit (param) {
    // 登录提交
    let res = await Api.getphoneusermsg(param.phone, param.vercode, false)
    if (res.status) {
      sessionStorage.setItem('UserID', res.UserID)
      sessionStorage.setItem('LoginUID', res.LoginUID)
      sessionStorage.setItem('User_Name', res.UserName)
      sessionStorage.setItem('Full_Name', res.FullName)
      sessionStorage.setItem('avatar', res.icon || '')
      sessionStorage.setItem('dataM', res.dataM ? 'true' : '')
      sessionStorage.setItem('debug', res.debug || '')
      sessionStorage.setItem('role_id', res.role_id || '')
      sessionStorage.setItem('departmentcode', res.departmentcode || '')
      sessionStorage.setItem('organization', res.organization || '')
      sessionStorage.setItem('mk_user_type', res.mk_user_type || '')
      
      if (res.paas_externalDatabase) {
        sessionStorage.setItem('externalDatabase', res.paas_externalDatabase)
        window.GLOB.externalDatabase = `[${res.paas_externalDatabase}]..`
      }

      localStorage.setItem(_href + 'lang', param.lang || 'zh-CN')
      sessionStorage.setItem('lang', param.lang || 'zh-CN')

      sessionStorage.removeItem('visitorUserID')
      sessionStorage.removeItem('visitorLoginUID')

      this.props.history.replace('/main')
    } else if (res.ErrCode === 'Need_Get_Appkey' && options.sysType === 'SSO') {
      message.warning('应用尚未创建，请向云端同步应用！')

      this.setState({
        isDisabled: false,
        syncApp: true
      })
    } else {
      message.warning(res.message)
      this.setState({
        isDisabled: false
      })
    }
  }

  authLogin = (appid, openid, memberid, scanId) => {
    Api.getTouristMsg('mk', appid, openid, memberid, scanId).then(res => {
      if (res.status) {
        sessionStorage.setItem('UserID', res.UserID)
        sessionStorage.setItem('LoginUID', res.LoginUID)
        sessionStorage.setItem('User_Name', res.UserName)
        sessionStorage.setItem('Full_Name', res.FullName)
        sessionStorage.setItem('avatar', res.icon || '')
        sessionStorage.setItem('dataM', res.dataM ? 'true' : '')
        sessionStorage.setItem('debug', res.debug || '')
        sessionStorage.setItem('role_id', res.role_id || '')
        sessionStorage.setItem('departmentcode', res.departmentcode || '')
        sessionStorage.setItem('organization', res.organization || '')
        sessionStorage.setItem('mk_user_type', res.mk_user_type || '')

        if (res.paas_externalDatabase) {
          sessionStorage.setItem('externalDatabase', res.paas_externalDatabase)
          window.GLOB.externalDatabase = `[${res.paas_externalDatabase}]..`
        }
  
        sessionStorage.removeItem('visitorUserID')
        sessionStorage.removeItem('visitorLoginUID')
  
        this.props.history.replace('/main')
      } else if (res.ErrCode === 'Need_Get_Appkey' && options.sysType === 'SSO') {
        message.warning('应用尚未创建，请向云端同步应用！')
  
        this.setState({
          isDisabled: false,
          syncApp: true
        })
      } else {
        message.warning(res.message)
        this.setState({
          isDisabled: false
        })
      }
    })
  }

  jsonp(url) {
    return new Promise((resolve, reject) => {
      window.jsonCallBack = (result) => {
        resolve(result)
      }

      const JSONP = document.createElement('script')
      JSONP.type = 'text/javascript'
      JSONP.src = url

      document.getElementsByTagName('head')[0].appendChild(JSONP)

      setTimeout(() => {
        document.getElementsByTagName('head')[0].removeChild(JSONP)
      },500)
    })
}

  componentDidMount () {
    // md5("/ws/location/v1/ip?callback=callbackFunction&key=key&output=jsonp secret key")
    // md5("/ws/location/v1/ip?callback=callbackFunction&key=BA7BZ-4QB65-LFCIA-QPDA6-4G6O7-MJB4Q&output=jsonpuThL4ZM3XOj642ksEQh76tyHFjh4")

    // 获取ip及城市信息
    // let ipurl = window.atob('aHR0cHM6Ly9lcGMubWs5a$mkC5jbi93ZWJhcGkvaXBsb2M='.replace(/\$mk/ig, ''))
    // Api.directRequest(ipurl, 'get', null, 'true').then(res => {
    //   if (!res || !res.ip) return
    //   sessionStorage.setItem('ipAddress', res.ip)
    // })

    setTimeout(() => {
      Api.delCacheConfig()
    }, 50)
    if (window.GLOB.filter) {
      let view = document.getElementById('mk-login-view')
      
      if (view) {
        view.style.filter = 'grayscale(100%)'
      }
    }

    if (sessionStorage.getItem('loginError')) {
      try {
        let res = JSON.parse(sessionStorage.getItem('loginError'))
        console.info(res.url)
        console.info(res.request)
        console.info(res.response)

        let result = JSON.parse(res.response)

        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
      } catch (e) {}
      
      setTimeout(() => {
        sessionStorage.removeItem('loginError')
      }, 2000)
    }

    const _addressUrl = _href + 'queryAddress'

    if (localStorage.getItem(_addressUrl) === 'true') {
      this.queryAddress()
    }

    const timeStamp = new Date().getTime()
    const _authUrl = _href + 'AuthCode'

    let authCode = localStorage.getItem(_authUrl)
    let _s = md5('mksoft' + moment().format('YYYYMMDD'))
    authCode = authCode ? authCode.split(',') : []
    let index = authCode.findIndex(key => key === _s)
    let license = false

    if (window.GLOB.licenseKey) {
      if (window.GLOB.licenseKey.length !== 20) {
        Modal.warning({
          title: `本地授权码已失效，请联系管理员。`,
          okText: '知道了'
        })
      } else {
        let key = md5(window.GLOB.appId + 'minke_software' + window.GLOB.appkey).toUpperCase().substr(-6)
  
        let key1 = window.GLOB.licenseKey.substring(0, 6)
        let key2 = window.GLOB.licenseKey.substring(6, 14)
        let key3 = window.GLOB.licenseKey.substring(14)
        let key4 = md5(key1 + key2).toUpperCase().substr(-6)
  
        if (key === key1 && key3 === key4) {
          let last = window.GLOB.appkey[window.GLOB.appkey.length - 1]
          let offset = 0
          let keys = {}
      
          if (!isNaN(+last)) {
            offset = (10 - last) % 10
          } else {
            last = last.charCodeAt() - 65
            offset = (30 - last) % 10
          }
      
          for (let i = 0; i < 26; i++) {
            let char = String.fromCharCode(65 + i)
            keys[char] = (offset + i) % 10
          }
  
          let lictime = key2.replace(/[A-Z]/ig, (s) => {
            return keys[s]
          })
          lictime = lictime.slice(0,4) + '/' + lictime.slice(4,6) + '/' + lictime.slice(6)
          lictime = Math.floor(new Date(lictime).getTime() / 86400000)
          let curtime = Math.floor(new Date().getTime() / 86400000)
          let licday = lictime - curtime
          let _mindUrl = _href + 'notMind'

          if (licday < 0 || isNaN(licday)) {
            Modal.warning({
              title: '本地授权码已失效，请联系管理员。',
              okText: '知道了'
            })
          } else if (licday > 20) {
            license = true
            localStorage.removeItem(_mindUrl)
          } else {
            license = true
  
            if (localStorage.getItem(_mindUrl) !== 'false') {
              Modal.confirm({
                title: `距离授权到期还剩${licday}天，请联系管理员。`,
                okText: '不在提醒',
                cancelText: '关闭',
                onOk() {
                  localStorage.setItem(_mindUrl, 'false')
                },
                onCancel() {},
              })
            }
          }
        } else {
          Modal.warning({
            title: '本地授权码已失效，请联系管理员。',
            okText: '知道了'
          })
        }
      }
    }

    if ((!window.GLOB.licenseKey && index > -1) || license) {
      this.setState({
        auth: true
      })
    }

    if (!window.GLOB.licenseKey && (index === -1 || index > 5)) {
      let _appId = window.GLOB.appId
  
      if (options.sysType === 'cloud') { // 云端使用系统配置appid
        _appId = options.caId
      }
  
      let str = md5('MK19' + _appId + timeStamp)
      let _rduri = window.atob('aHR0cHM6Ly9lcGMubWs5aC5$mkjbi93ZWJhcGkvZG9zdGFycw=='.replace(/\$mk/ig, ''))
      let _func = window.atob('c0VtcG93ZXJDbG91$mkZF9HZXRfTGlua1VybA=='.replace(/\$mk/ig, ''))
      let _id = window.atob('YmgwYmFwYWJ0ZDQ1ZXBz$mkZ3JhNzlzZWdiY2g2YzFpYms='.replace(/\$mk/ig, ''))
  
      let param = {
        func: _func,
        AppID: _appId,
        TimeStamp: timeStamp,
        appkey: window.GLOB.appkey,
        userid: _id,
        LoginUID: _id,
        nonc: Utils.getuuid()
      }

      // param = {
      //   func: _func,
      //   VerificationCode: '授权码',
      //   TimeStamp: timeStamp,
      //   userid: _id,
      //   LoginUID: _id,
      //   nonc: Utils.getuuid()
      // }

      let keys = Object.keys(param).sort()
      let values = ''
      keys.forEach(key => {
        values += key + param[key]
      })
      param.sign = md5(values)
      param.t = new Date().getTime()
  
      Api.directRequest(_rduri + '/' + _func, 'post', param, 'true').then(res => {
        if (res.status) {
          if (res.EPC === str) {
            let box = []
            for (let i = 0; i < 15; i++) {
              let s = 'mksoft' + moment().add(i, 'days').format('YYYYMMDD')
              box.push(md5(s))
            }
            box = box.join(',')
            localStorage.setItem(_authUrl, box)
    
            this.setState({
              auth: true
            })

            if (res.warning_day && res.warning_day < 20) {
              Modal.warning({
                title: `距离授权到期还剩${res.warning_day}天，请联系管理员。`,
                okText: '知道了'
              })
            }
          } else {
            localStorage.removeItem(_authUrl)
            this.setState({
              auth: false
            })
          }

          if (res.query_address !== 'true') {
            localStorage.setItem(_addressUrl, 'false')
          } else {
            localStorage.setItem(_addressUrl, 'true')
            this.queryAddress()
          }
        } else if (res.ErrCode === 'N') {
          localStorage.removeItem(_authUrl)
          this.setState({
            auth: false,
            authError: res.message
          })
        }
      }, () => {
        if (index === -1 || index > 10) {
          this.setState({
            auth: false,
            authError: '网络错误导致系统授权失败，请联系管理员。'
          })
        }
      })
    }

    Api.getTouristMsg().then(result => {
      if (result.status) {
        if (result.website && process.env.NODE_ENV === 'production') {
          let website = result.website.replace(/http(s)?:\/\/|\/$/ig, '').toLowerCase()
          let current = window.GLOB.baseurl.replace(/http(s)?:\/\/|\/$/ig, '').toLowerCase()

          if (website !== current) {
            try {
              window.history.replaceState(null, null, result.website.replace(/\/$/ig, '') + '/admin/index.html#/login')
              window.location.reload()
            } catch(e) {
              window.location.href = result.website.replace(/\/$/ig, '') + '/admin/index.html#/login'
            }
            return
          }
        }
        
        sessionStorage.setItem('visitorUserID', result.UserID || '')
        sessionStorage.setItem('visitorLoginUID', result.LoginUID || '')

        if (result.UserID && result.LoginUID) {
          this.setState({touristLogin: true})
        }

        // 获取系统信息
        let _param = {
          func: 's_Get_style',
          TypeCharOne: 'PC',
          LText: `select '${window.GLOB.appkey}'`,
        }

        _param.userid = result.UserID
        _param.LoginUID = result.LoginUID
        _param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
        _param.secretkey = Utils.encrypt(_param.LText, _param.timestamp)

        Api.getSystemConfig(_param).then(res => {
          if (res.status) {
            // positecgroup
            res.Banner = res.Banner ? res.Banner.replace(/:8080/ig, '').replace(/http:/ig, 'https:') : ''
            res.doclogo = res.doclogo ? res.doclogo.replace(/:8080/ig, '').replace(/http:/ig, 'https:') : ''
            res.indexlogo = res.indexlogo ? res.indexlogo.replace(/:8080/ig, '').replace(/http:/ig, 'https:') : ''
            res.loginlogo = res.loginlogo ? res.loginlogo.replace(/:8080/ig, '').replace(/http:/ig, 'https:') : ''

            if (!['shutter', 'linkage_navigation', 'linkage', 'menu_board', 'menu_board_navigation'].includes(res.menu_type)) {
              res.menu_type = 'shutter'
            }

            let _url = _href + 'system'
            let systemMsg = {
              favicon: res.titlelogo || '',
              platTitle: res.titleName || '',
              platName: res.SysName || '',
              bgImage: res.Banner || '',
              loginlogo: res.loginlogo || '',
              copyRight: res.CopyRightYear && res.CopyRightOrg ? `Copyright©${res.CopyRightYear}    所有相关版权归    ${res.CopyRightOrg}` : '',
              ICP: res.ICP || '',
              mainlogo: res.indexlogo || '',
              doclogo: res.doclogo || '',
              style: res.CSS || '',
              showline: res.split_line_show || 'true',
              webSite: res.WebSite || '',
              navBar: res.menu_type, // shutter 百叶窗、linkage_navigation 联动菜单、linkage 联动菜单_无导航栏、menu_board 菜单面板、menu_board_navigation 菜单面板_标签页
              app_version: res.app_version
            }

            let level = res.pwd_level || ''

            if (level && !['letter_num', 'char_num', 'char_num_90', 'char_num_90_sms'].includes(level)) {
              level = ''
            }

            localStorage.setItem(_href + 'pwdlevel', level)

            if (res.srcid) {
              localStorage.setItem(_href + 'srcId', res.srcid)
            } else {
              localStorage.removeItem(_href + 'srcId')
            }

            sessionStorage.setItem('home_background', res.index_background_color || '')

            let seconds = 0
            if (res.sys_datetime) {
              seconds = Math.floor((new Date(res.sys_datetime).getTime() - new Date().getTime()) / 1000)
            }
            sessionStorage.setItem('sys_time_shim', isNaN(seconds) ? 0 : seconds)

            // url标题
            document.title = systemMsg.platTitle

            try {
              localStorage.setItem(_url, window.btoa(window.encodeURIComponent(JSON.stringify(systemMsg))))
            } catch (e) {
              localStorage.removeItem(_url)
            }

            let _loginurl = _href + 'loginways'
            let login_ways = []
            let login_types = []
            if (res.login_ways && res.login_ways.length > 0) {
              res.login_ways.forEach(item => {
                // 短信验证码登录，必须设置短信Id
                if (item.way_no === 'sms_vcode' && !item.sms_id) return
                if (login_types.includes(item.way_no)) return

                login_types.push(item.way_no)

                login_ways.push({
                  type: item.way_no,
                  smsId: item.sms_id
                })

                if (item.way_no === 'sms_vcode' && item.sms_id) {
                  sessionStorage.setItem('mk_sms_id', item.sms_id)
                }
              })
            } else {
              login_ways.push({
                type: 'uname_pwd',
                smsId: ''
              })
            }

            try {
              localStorage.setItem(_loginurl, window.btoa(window.encodeURIComponent(JSON.stringify(login_ways))))
            } catch (e) {
              localStorage.removeItem(_loginurl)
            }
            
            this.setState({
              loginWays: login_ways,
              langList: res.lang_data || [],
              ...systemMsg
            })

            window.GLOB.mainlogo = systemMsg.mainlogo
            window.GLOB.style = systemMsg.style
            window.GLOB.navBar = systemMsg.navBar
            window.GLOB.appVersion = systemMsg.app_version
        
            if (window.GLOB.style && styles[window.GLOB.style]) {
              document.body.className = styles[window.GLOB.style] + ' ' + (res.split_line_show === 'false' ? 'hidden-split-line' : '')
            }

            if (res.titlelogo && window.GLOB.favicon !== res.titlelogo) {
              let link = document.querySelector("link[rel*='icon']") || document.createElement('link')
              link.type = 'image/x-icon'
              link.rel = 'shortcut icon'
              link.href = res.titlelogo
              document.getElementsByTagName('head')[0].appendChild(link)
            }

            let memberLevel = res.member_level

            if (typeof(memberLevel) === 'number' && memberLevel > 10 && parseInt(memberLevel / 10) * 10 === memberLevel) {
              sessionStorage.setItem('Member_Level', md5('mksoft' + window.GLOB.appkey + new Date().getFullYear() + new Date().getMonth() + memberLevel))
              window.GLOB.memberLevel = memberLevel
            }

            // positecgroup
            if (res.users_upt === 'true' && window.GLOB.systemType === 'production') {
              Api.genericInterface({
                func: 's_Get_local_u_deleted',
                users_upt_date: res.users_upt_date,
                userid: result.UserID,
                LoginUID: result.LoginUID
              }).then(localres => {
                if (!localres.status) return
                Api.getSystemConfig({
                  func: 's_get_sso_u_create',
                  user_ids: localres.user_ids,
                  user_ids_local: localres.user_ids_local,
                  users_upt_date: localres.users_upt_date,
                  userid: result.UserID,
                  LoginUID: result.LoginUID
                }).then(ssores => {
                  if (!ssores.status) return
                  Api.genericInterface({
                    func: 's_get_local_u_create',
                    user_ids_local: ssores.user_ids_local,
                    userid: result.UserID,
                    LoginUID: result.LoginUID
                  })
                })
              })
            }
          } else {
            message.warning(res.message)
          }
        })
      } else {
        message.warning(result.message)
      }
    })
    
    let loginWays = localStorage.getItem(_href + 'loginways')
    if (loginWays) {
      try {
        loginWays = JSON.parse(window.decodeURIComponent(window.atob(loginWays)))
      } catch (e) {
        loginWays = null
      }

      this.setState({
        loginWays: loginWays
      })
    }
  }

  queryAddress = () => {
    window.callbackFunction = (res) => {
      if (res.result && res.result.ad_info) {
        sessionStorage.setItem('nation', res.result.ad_info.nation || '')
        sessionStorage.setItem('province', res.result.ad_info.province || '')
        sessionStorage.setItem('city', res.result.ad_info.city || '')
        sessionStorage.setItem('district', res.result.ad_info.district || '')
        sessionStorage.setItem('address', res.result.ad_info.address || '')
        sessionStorage.setItem('ipAddress', res.result.ip || '')
      }
    }

    const JSONP = document.createElement('script')
    JSONP.type = 'text/javascript'
    JSONP.src = 'https://apis.map.qq.com/ws/location/v1/ip?callback=callbackFunction&key=BA7BZ-4QB65-LFCIA-QPDA6-4G6O7-MJB4Q&output=jsonp&sig=3e5ebecb324ba266bf80014dcc8380db'
    document.getElementsByTagName('head')[0].appendChild(JSONP)

    setTimeout(() => {
      document.getElementsByTagName('head')[0].removeChild(JSONP)
    },500)
  }

  syncSubmit = () => {
    this.setState({
      syncing: true
    })

    let param = {
      func: 's_get_app_from_cloud',
      UserName: '',
      Password: '',
      systemType: options.sysType,
      Type: 'X',
      debug: 'Y'
    }

    param.appkey = window.GLOB.appkey || ''

    if (options.cloudServiceApi) {
      param.rduri = options.cloudServiceApi.replace('dostars', 'dostar')
    }

    this.logincloudRef.handleConfirm().then(result => {
      param.UserName = result.cloudusername
      param.Password = result.cloudpassword

      param.Password = Utils.formatOptions(param.Password)

      Api.dostarInterface(param).then(res => {
        if (res.status) {
          let _param = {
            func: 's_sDataDictb_TBBack',
            LTextOut: res.LTextOut,
            lid: res.Lid,
            Appkey: window.GLOB.appkey || ''
          }

          _param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
          _param.secretkey = Utils.encrypt(_param.LTextOut, _param.timestamp)
          _param.open_key = Utils.encryptOpenKey(_param.secretkey, _param.timestamp)

          Api.dostarInterface(_param).then(response => {
            if (response.status) {
              message.success('应用创建成功。')
              this.setState({
                syncing: false,
                syncApp: false
              })
            } else {
              this.setState({
                syncing: false
              })
              message.warning(response.message)
            }
          })

        } else {
          this.setState({
            syncing: false
          })
          message.warning(res.message)
        }
      })
    })
  }

  codeSubmit = () => {
    const { verify } = this.state
    this.codeRef.handleConfirm().then(res => {
      this.setState({
        codeLoading: true
      })
      Api.verifycode({...verify, code: res.code}).then(result => {
        this.setState({
          codeVisible: false,
          codeLoading: false
        })

        if (result.verifyResult === 'success') {
          let level = localStorage.getItem(_href + 'pwdlevel')
          if (level) {
            let visible = false
            let tip = '密码强度不够，请修改密码！'
            if (verify.$password.length < 8) {
              visible = true
            } else if (level === 'letter_num' && /^([^0-9]*|[^a-zA-Z]*)$/.test(verify.$password)) {
              visible = true
            } else if ((level === 'char_num' || level === 'char_num_90') && /^([^0-9]*|[^a-zA-Z]*|[^!@#$%^&*()_]*)$/.test(verify.$password)) {
              visible = true
            } else if (level === 'char_num_90' && verify.modifydate) {
              let s = (new Date().getTime() - new Date(verify.modifydate).getTime()) / (1000 * 24 * 60 * 60)
              if (!isNaN(s) && s > 90) {
                visible = true
                tip = '您已90天未修改密码，请更换密码后使用！'
              }
            }

            if (visible) {
              message.warning(tip)
              this.setState({
                isDisabled: false,
                visible: true
              })
              return
            }
          }

          this.props.history.replace('/main')
        } else {
          notification.warning({
            top: 92,
            message: '验证码无效，请重新登录！',
            duration: 5
          })
        }
      })
    }, () => {})
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  render () {
    const { lineColor, loginlogo, bgImage, copyRight, webSite, ICP, loginWays, touristLogin } = this.state

    return (
      <div className="login-container" id="mk-login-view">
        <div className="logo" style={lineColor ? {borderColor: lineColor} : {}}>
          {loginlogo ? <img src={loginlogo} alt=""/> : null}
        </div>
        <div className="login-middle" style={bgImage ? {backgroundImage: 'url(' + bgImage + ')'} : null}>
          {loginWays ? <LoginForm
            platName={this.state.platName}
            dict={this.state.dict}
            auth={this.state.auth}
            authError={this.state.authError}
            touristLogin={touristLogin}
            loginWays={loginWays}
            lang={this.state.selectedlang}
            langList={this.state.langList}
            isDisabled={this.state.isDisabled}
            changelang={(value) => this.changelang(value)}
            handleSubmit={() => this.handleSubmit()}
            authLogin={this.authLogin}
            wrappedComponentRef={(inst) => this.loginformRef = inst}
          /> : null}
          <div className="login-bottom">
            {webSite && copyRight ?
              <a target="_blank" rel="noopener noreferrer" href={webSite} dangerouslySetInnerHTML={{ __html: copyRight.replace(/\s/ig, '&nbsp;') }}></a> :
              <p dangerouslySetInnerHTML={{ __html: copyRight ? copyRight.replace(/\s/ig, '&nbsp;') : '' }}></p>
            }
            {ICP ? <p dangerouslySetInnerHTML={{ __html: ICP.replace(/\s/ig, '&nbsp;') }}></p> : null}
          </div>
        </div>
        {/* 编辑状态登录 */}
        <Modal
          title={this.state.dict['login.sync.cloud']}
          okText={this.state.dict['login.ok']}
          cancelText={this.state.dict['login.cancel']}
          visible={this.state.syncApp}
          onOk={this.syncSubmit}
          maskClosable={false}
          className="sync-cloud-application"
          width={'430px'}
          confirmLoading={this.state.syncing}
          onCancel={() => {this.setState({ syncApp: false, syncing: false })}}
          destroyOnClose
        >
          <LoginCloudForm handleSubmit={() => this.syncSubmit()} wrappedComponentRef={(inst) => this.logincloudRef = inst}/>
        </Modal>
        {/* 修改密码 */}
        <Resetpwd />
        {/* 二次验证 */}
        <Modal
          title="二次验证"
          okText={this.state.dict['login.ok']}
          cancelText={this.state.dict['login.cancel']}
          visible={this.state.codeVisible}
          onOk={this.codeSubmit}
          maskClosable={false}
          confirmLoading={this.state.codeLoading}
          onCancel={() => this.setState({codeVisible: false, codeLoading: false})}
          destroyOnClose
        >
          <LoginCodeForm verify={this.state.verify} wrappedComponentRef={(inst) => this.codeRef = inst} handleSubmit={this.codeSubmit}/>
        </Modal>
      </div>
    )
  }
}

export default Login